/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::ArmarXObjects::ControlTargetHolonomicPlatformVelocity
 * @author     Raphael ( ufdrv at student dot kit dot edu )
 * @date       2017
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#ifndef _ARMARX_LIB_RobotAPI_ControlTargetHolonomicPlatformVelocity_H
#define _ARMARX_LIB_RobotAPI_ControlTargetHolonomicPlatformVelocity_H

#include "ControlTargetBase.h"
#include <ArmarXCore/observers/variant/Variant.h>

namespace armarx
{
    /**
    * @defgroup Library-RobotUnit RobotUnit
    * @ingroup RobotAPI
    * A description of the library RobotUnit.
    *
    * @class ControlTargetHolonomicPlatformVelocity
    * @ingroup Library-RobotUnit
    * @brief Brief description of class ControlTargetHolonomicPlatformVelocity.
    *
    * Detailed description of class ControlTargetHolonomicPlatformVelocity.
    */
    class ControlTargetHolonomicPlatformVelocity: public ControlTargetBase
    {
    public:
        float velocityX = ControllerConstants::ValueNotSetNaN;
        float velocityY = ControllerConstants::ValueNotSetNaN;
        float velocityRotation = ControllerConstants::ValueNotSetNaN;
        virtual const std::string& getControlMode() const override
        {
            return ControlModes::HolonomicPlatformVelocity;
        }
        virtual void reset() override
        {
            velocityX = ControllerConstants::ValueNotSetNaN;
            velocityY = ControllerConstants::ValueNotSetNaN;
            velocityRotation = ControllerConstants::ValueNotSetNaN;
        }
        virtual bool isValid() const override
        {
            return std::isfinite(velocityX) && std::isfinite(velocityY) && std::isfinite(velocityRotation);
        }
        virtual std::map<std::string, VariantBasePtr>   toVariants(const IceUtil::Time& timestamp) const override
        {
            return
            {
                {"velocityX",        {new TimedVariant{velocityX       ,timestamp}}},
                {"velocityY",        {new TimedVariant{velocityY       ,timestamp}}},
                {"velocityRotation", {new TimedVariant{velocityRotation,timestamp}}},
            };
        }

        DETAIL_ControlTargetBase_DEFAULT_METHOD_IMPLEMENTATION
    };
}

#endif
