/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::ArmarXObjects::LVL0Controller
 * @author     Raphael ( ufdrv at student dot kit dot edu )
 * @date       2017
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#ifndef _ARMARX_LIB_RobotAPI_LVL0Controller_H
#define _ARMARX_LIB_RobotAPI_LVL0Controller_H

#include "../ControlTargets/ControlTargetBase.h"
#include "../util.h"

#include <memory>
#include <atomic>

namespace armarx
{
    class ControlDevice;

    class LVL0Controller
    {
    public:
        virtual ~LVL0Controller() = default;

        virtual ControlTargetBase* getControlTarget() = 0;

        virtual const ControlTargetBase* getControlTarget() const
        {
            return const_cast<const ControlTargetBase*>(const_cast<LVL0Controller*>(this)->getControlTarget());
        }

        virtual void rtResetTarget()
        {
            getControlTarget()->reset();
        }

        virtual bool rtIsTargetVaild() const
        {
            return getControlTarget()->isValid();
        }

        virtual const std::string& getControlMode() const
        {
            return getControlTarget()->getControlMode();
        }
        ControlDevice& getParent() const;

    protected:
        ControlDevice& rtGetParent() const
        {
            return *parent;
        }
        //called by the owning ControlDevice
        /// @brief called when this LVL0Controller is run
        virtual void rtRun(const IceUtil::Time& sensorValuesTimestamp, const IceUtil::Time& timeSinceLastIteration) = 0;
        /// @brief called when this LVL0Controller is activated
        virtual void rtPreActivateController()
        {

        }
        virtual void rtPostDeactivateController()
        {

        }
        /// @brief called when this LVL0Controller is deactivated
    private:
        friend class ControlDevice;
        std::atomic_bool activated {false};
        ControlDevice* parent {nullptr};
        void rtActivate()
        {
            rtPreActivateController();
            activated = true;
        }
        void rtDeactivate()
        {
            activated = false;
            rtPostDeactivateController();
            rtResetTarget();
        }
    };
}
#endif
