/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::ArmarXObjects::LVL1HolonomicPlatformUnitVelocityPassThroughController
 * @author     Raphael ( ufdrv at student dot kit dot edu )
 * @date       2017
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#include "LVL1HolonomicPlatformUnitVelocityPassThroughController.h"

#include <ArmarXCore/core/exceptions/local/ExpressionException.h>

using namespace armarx;

LVL1HolonomicPlatformUnitVelocityPassThroughController::LVL1HolonomicPlatformUnitVelocityPassThroughController(LVL1ControllerDescriptionProviderInterfacePtr prov, LVL1ControllerConfigPtr config, const VirtualRobot::RobotPtr&)
{
    LVL1HolonomicPlatformUnitVelocityPassThroughControllerConfigPtr cfg = LVL1HolonomicPlatformUnitVelocityPassThroughControllerConfigPtr::dynamicCast(config);
    ARMARX_CHECK_EXPRESSION_W_HINT(cfg, "The provided config has the wrong type! The type is " << config->ice_id());

    target = prov->getControlTarget(cfg->platformName, ControlModes::HolonomicPlatformVelocity)->asA<ControlTargetHolonomicPlatformVelocity>();
    ARMARX_CHECK_EXPRESSION_W_HINT(target, "The actuator " << cfg->platformName << " has no control mode " << ControlModes::HolonomicPlatformVelocity);

    initialSettings.velocityX        = cfg->initialVelocityX;
    initialSettings.velocityY        = cfg->initialVelocityY;
    initialSettings.velocityRotation = cfg->initialVelocityRotation;
    reinitTripleBuffer(initialSettings);
}

void LVL1HolonomicPlatformUnitVelocityPassThroughController::rtRun(const IceUtil::Time&, const IceUtil::Time&)
{
    target->velocityX        = rtGetControlStruct().velocityX;
    target->velocityY        = rtGetControlStruct().velocityY;
    target->velocityRotation = rtGetControlStruct().velocityRotation;
}

void LVL1HolonomicPlatformUnitVelocityPassThroughController::setVelocites(float velocityX, float velocityY,
        float velocityRotation)
{
    LockGuardType guard {controlDataMutex};
    getWriterControlStruct().velocityX        = velocityX;
    getWriterControlStruct().velocityY        = velocityY;
    getWriterControlStruct().velocityRotation = velocityRotation;
    writeControlStruct();
}

LVL1ControllerRegistration<LVL1HolonomicPlatformUnitVelocityPassThroughController>
registrationLVL1HolonomicPlatformUnitVelocityPassThroughController("LVL1HolonomicPlatformUnitVelocityPassThroughController");
