/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::ArmarXObjects::LVL1HolonomicPlatformUnitVelocityPassThroughController
 * @author     Raphael ( ufdrv at student dot kit dot edu )
 * @date       2017
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#ifndef _ARMARX_LIB_RobotAPI_LVL1HolonomicPlatformUnitVelocityPassThroughController_H
#define _ARMARX_LIB_RobotAPI_LVL1HolonomicPlatformUnitVelocityPassThroughController_H

#include <VirtualRobot/Robot.h>

#include "LVL1Controller.h"
#include "../ControlTargets/ControlTargetHolonomicPlatformVelocity.h"
#include "../util.h"

namespace armarx
{

    TYPEDEF_PTRS_HANDLE(LVL1HolonomicPlatformUnitVelocityPassThroughControllerConfig);
    class LVL1HolonomicPlatformUnitVelocityPassThroughControllerConfig: virtual public LVL1ControllerConfig
    {
    public:
        std::string platformName;
        float initialVelocityX;
        float initialVelocityY;
        float initialVelocityRotation;
    };

    TYPEDEF_PTRS_HANDLE(LVL1HolonomicPlatformUnitVelocityPassThroughController);
    class LVL1HolonomicPlatformUnitVelocityPassThroughControllerControlData
    {
    public:
        float velocityX = 0;
        float velocityY = 0;
        float velocityRotation = 0;
    };

    TYPEDEF_PTRS_HANDLE(LVL1HolonomicPlatformUnitVelocityPassThroughController);
    class LVL1HolonomicPlatformUnitVelocityPassThroughController:
        virtual public LVL1ControllerWithTripleBuffer<LVL1HolonomicPlatformUnitVelocityPassThroughControllerControlData>
    {
    public:
        LVL1HolonomicPlatformUnitVelocityPassThroughController(LVL1ControllerDescriptionProviderInterfacePtr prov, LVL1ControllerConfigPtr config, const VirtualRobot::RobotPtr&);

        virtual void rtRun(const IceUtil::Time&, const IceUtil::Time&) override;

        //for the platform unit
        void setVelocites(float velocityX, float velocityY, float velocityRotation);

        //ice interface
        virtual std::string getClassName(const Ice::Current& = GlobalIceCurrent) const override
        {
            return "LVL1HolonomicPlatformUnitVelocityPassThroughController";
        }
    protected:
        virtual void onInitComponent() override {}
        virtual void onConnectComponent() override {}

        ControlTargetHolonomicPlatformVelocity* target;
        LVL1HolonomicPlatformUnitVelocityPassThroughControllerControlData initialSettings;
    };
}
#endif
