/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::ArmarXObjects::LVL1KinematicUnitPassThroughController
 * @author     Raphael ( ufdrv at student dot kit dot edu )
 * @date       2017
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#include "LVL1KinematicUnitPassThroughController.h"
using namespace armarx;

LVL1ControllerRegistration<LVL1KinematicUnitPassThroughController> registrationControllerLVL1KinematicUnitPassThroughController("LVL1KinematicUnitPassThroughController");

LVL1KinematicUnitPassThroughController::LVL1KinematicUnitPassThroughController(LVL1ControllerDescriptionProviderInterfacePtr prov, const LVL1ControllerConfigPtr& config, const VirtualRobot::RobotPtr&)
{
    LVL1KinematicUnitPassThroughControllerConfigPtr cfg = LVL1KinematicUnitPassThroughControllerConfigPtr::dynamicCast(config);
    ARMARX_CHECK_EXPRESSION_W_HINT(cfg, "The provided config has the wrong type! The type is " << config->ice_id());

    const SensorValueBase* sv = prov->getSensorValue(cfg->deviceName);
    ControlTargetBase* ct = prov->getControlTarget(cfg->deviceName, cfg->controlMode);
    //get sensor
    if (cfg->controlMode == ControlModes::Position1DoF)
    {
        ARMARX_CHECK_EXPRESSION(sv->asA<SensorValue1DoFActuatorPosition>());
        sensor = &(sv->asA<SensorValue1DoFActuatorPosition>()->position);
        ARMARX_CHECK_EXPRESSION(ct->asA<ControlTarget1DoFActuatorPosition>());
        target = &(ct->asA<ControlTarget1DoFActuatorPosition>()->position);
        sensorToControlOnActivateFactor = 1;
    }
    else if (cfg->controlMode == ControlModes::Velocity1DoF)
    {
        ARMARX_CHECK_EXPRESSION(sv->asA<SensorValue1DoFActuatorVelocity>());
        sensor = &(sv->asA<SensorValue1DoFActuatorVelocity>()->velocity);
        ARMARX_CHECK_EXPRESSION(ct->asA<ControlTarget1DoFActuatorVelocity>());
        target = &(ct->asA<ControlTarget1DoFActuatorVelocity>()->velocity);
        sensorToControlOnActivateFactor = 1;
    }
    else if (cfg->controlMode == ControlModes::Torque1DoF)
    {
        ARMARX_CHECK_EXPRESSION(sv->asA<SensorValue1DoFActuatorTorque>());
        sensor = &(sv->asA<SensorValue1DoFActuatorTorque>()->torque);
        ARMARX_CHECK_EXPRESSION(ct->asA<ControlTarget1DoFActuatorTorque>());
        target = &(ct->asA<ControlTarget1DoFActuatorTorque>()->torque);
        sensorToControlOnActivateFactor = 0;
    }
    else
    {
        throw InvalidArgumentException {"Unsupported control mode: " + cfg->controlMode};
    }
}
