/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::ArmarXObjects::LVL1KinematicUnitPassThroughController
 * @author     Raphael ( ufdrv at student dot kit dot edu )
 * @date       2017
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#ifndef _ARMARX_LIB_RobotAPI_LVL1KinematicUnitPassThroughController_H
#define _ARMARX_LIB_RobotAPI_LVL1KinematicUnitPassThroughController_H

#include <atomic>

#include <VirtualRobot/Robot.h>

#include <ArmarXCore/core/exceptions/local/ExpressionException.h>

#include "LVL1Controller.h"
#include "../RobotUnit.h"
#include "../SensorValues/SensorValue1DoFActuator.h"

#include "../ControlTargets/ControlTarget1DoFActuator.h"

namespace armarx
{
    TYPEDEF_PTRS_HANDLE(LVL1KinematicUnitPassThroughControllerConfig);
    class LVL1KinematicUnitPassThroughControllerConfig : virtual public LVL1ControllerConfig
    {
    public:
        std::string deviceName;
        std::string controlMode;
    };


    TYPEDEF_PTRS_HANDLE(LVL1KinematicUnitPassThroughController);
    class LVL1KinematicUnitPassThroughController:
        virtual public LVL1Controller
    {
    public:
        inline LVL1KinematicUnitPassThroughController(LVL1ControllerDescriptionProviderInterfacePtr prov, const LVL1ControllerConfigPtr& config, const VirtualRobot::RobotPtr&);

        inline virtual void rtRun(const IceUtil::Time&, const IceUtil::Time&) override
        {
            *target = control;
        }
        inline virtual void rtPreActivateController() override
        {
            ARMARX_IMPORTANT << "<<<<<<<<<<<<<<<<<<<<<< ACT " << this;
            control = (*sensor) * sensorToControlOnActivateFactor;
        }
        virtual void rtPostDeactivateController() override
        {
            ARMARX_IMPORTANT << "<<<<<<<<<<<<<<<<<<<<<< DEC " << this;
        }

        //ice interface
        inline virtual std::string getClassName(const Ice::Current& = GlobalIceCurrent) const override
        {
            return "LVL1KinematicUnitPassThroughController";
        }

        void set(float val)
        {
            control = val;
        }

    protected:
        std::atomic<float> control {0};
        float* target {nullptr};
        const float* sensor
        {
            nullptr
        };
        float sensorToControlOnActivateFactor {0};

        virtual void onInitComponent() override {}
        virtual void onConnectComponent() override {}
    };
}
#endif
