/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::RobotUnit
 * @author     Raphael ( ufdrv at student dot kit dot edu )
 * @date       2017
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#ifndef _ARMARX_UNIT_RobotAPI_RobotUnit_structs_H
#define _ARMARX_UNIT_RobotAPI_RobotUnit_structs_H

#include <memory>
#include <vector>
#include "SensorValues/SensorValueBase.h"
#include "ControlTargets/ControlTargetBase.h"

namespace armarx
{
    class LVL0Controller;
    class LVL1Controller;

    /// @brief Structure used by the RobotUnit to swap out sensor values and control targets
    struct SensorAndControl
    {
        //since this is not automatically propagated from a vector of noncopyable elements
        SensorAndControl(const SensorAndControl&) = delete;
        SensorAndControl& operator=(const SensorAndControl&) = delete;
        //since deleting ctors stops autogeneration for everything
        SensorAndControl() = default;
        SensorAndControl(SensorAndControl&&) = default;
        SensorAndControl& operator=(SensorAndControl &&) = default;

        IceUtil::Time sensorValuesTimestamp;
        IceUtil::Time timeSinceLastIteration;
        std::vector<std::unique_ptr<SensorValueBase>> sensors;
        std::vector<std::vector<std::unique_ptr<ControlTargetBase>>> control;
    };
    /// @brief Structure used by the RobotUnit to swap lists of lvl0 and lvl1 controllers
    struct LVL0AndLVL1Controllers
    {
        LVL0AndLVL1Controllers(std::size_t n = 0)
            :   lvl0Controllers(n),
                lvl1Controllers(n)
        {}
        std::vector<LVL0Controller*> lvl0Controllers;
        std::vector<LVL1Controller*> lvl1Controllers;
    };
}
#endif
