/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::ArmarXObjects::MetaWearIMU
 * @author     Lukas Kaul ( lukas dot s dot kaul at gmail dot com )
 * @date       2017
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#ifndef _ARMARX_COMPONENT_RobotAPI_MetaWearIMU_H
#define _ARMARX_COMPONENT_RobotAPI_MetaWearIMU_H


#include <ArmarXCore/core/Component.h>
#include <RobotAPI/interface/units/MetaWearIMUInterface.h>
#include <RobotAPI/interface/units/MetaWearIMU.h>

namespace armarx
{
    /**
     * @class MetaWearIMUPropertyDefinitions
     * @brief
     */
    class MetaWearIMUPropertyDefinitions:
        public armarx::ComponentPropertyDefinitions
    {
    public:
        MetaWearIMUPropertyDefinitions(std::string prefix):
            armarx::ComponentPropertyDefinitions(prefix)
        {
            //defineRequiredProperty<std::string>("PropertyName", "Description");
            defineOptionalProperty<std::string>("MetaWearPythonTopicName", "MetaWearPythonData", "Name of the topic provided by the python driver");
            defineOptionalProperty<std::string>("MetaWearTopicName", "MetaWearData", "Name of the MetaWear topic");
        }
    };

    /**
     * @defgroup Component-MetaWearIMU MetaWearIMU
     * @ingroup RobotAPI-Components
     * A description of the component MetaWearIMU.
     *
     * @class MetaWearIMU
     * @ingroup Component-MetaWearIMU
     * @brief Brief description of class MetaWearIMU.
     *
     * Detailed description of class MetaWearIMU.
     */
    class MetaWearIMU :
        virtual public armarx::Component,
        virtual public armarx::MetaWearIMUInterface
    {
    public:
        /**
         * @see armarx::ManagedIceObject::getDefaultName()
         */
        virtual std::string getDefaultName() const
        {
            return "MetaWearIMU";
        }

    protected:
        /**
         * @see armarx::ManagedIceObject::onInitComponent()
         */
        virtual void onInitComponent();

        /**
         * @see armarx::ManagedIceObject::onConnectComponent()
         */
        virtual void onConnectComponent();

        /**
         * @see armarx::ManagedIceObject::onDisconnectComponent()
         */
        virtual void onDisconnectComponent();

        /**
         * @see armarx::ManagedIceObject::onExitComponent()
         */
        virtual void onExitComponent();

        /**
         * @see PropertyUser::createPropertyDefinitions()
         */
        virtual armarx::PropertyDefinitionsPtr createPropertyDefinitions();

        MetaWearIMUListenerPrx topicPrx;

        // MetaWearIMUInterface interface
    public:
        void reportIMUValues(const std::string& name, const MetaWearIMUDataExt& data_ext, const Ice::Current&);
    };
}

#endif
