/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * \package    RobotAPI::gui-plugins::RobotUnitPlugin
 * \author     Raphael ( ufdrv at student dot kit dot edu )
 * \date       2017
 * \copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#ifndef _ARMARX_RobotAPI_RobotUnitPlugin_RobotUnitLVL1ControllerClassesWidget_H
#define _ARMARX_RobotAPI_RobotUnitPlugin_RobotUnitLVL1ControllerClassesWidget_H

#include <mutex>

#include <QWidget>
#include <QLabel>
#include <QLineEdit>
#include <QHBoxLayout>
#include <QFormLayout>
#include <QPushButton>
#include <QTreeWidget>
#include <QTreeWidgetItem>
#include <QCheckBox>
#include <QComboBox>

#include <ArmarXGui/libraries/WidgetDescription/WidgetDescription.h>
#include <ArmarXGui/libraries/VariantWidget/VariantWidget.h>
#include <RobotAPI/interface/units/RobotUnit/RobotUnitInterface.h>

namespace Ui
{
    class RobotUnitLVL1ControllerClassesWidget;
}

namespace armarx
{

    class RobotUnitLVL1ControllerClassesWidgetEntry;

    class RobotUnitLVL1ControllerClassesWidget : public QWidget
    {
        Q_OBJECT

    public:
        explicit RobotUnitLVL1ControllerClassesWidget(QWidget* parent = 0);
        ~RobotUnitLVL1ControllerClassesWidget();

    public slots:
        virtual void lvl1ControllerClassAdded(std::string name);
        void reset(RobotUnitInterfacePrx ru);
    private slots:
        void filterUpdated();
    private:
        void add(const LVL1ControllerClassDescription& desc);
        std::map<std::string, RobotUnitLVL1ControllerClassesWidgetEntry*> entries;
        RobotUnitInterfacePrx robotUnit;
        mutable std::recursive_mutex mutex;
        Ui::RobotUnitLVL1ControllerClassesWidget* ui;

        QComboBox* filterCombination;

        QLineEdit* filterName;
        QCheckBox* filterNameActive;
        QCheckBox* filterNameInverted;

        QComboBox* filterRemoteCreation;
        QCheckBox* filterRemoteCreationActive;
    };

    //helper
    class RobotUnitLVL1ControllerClassesWidgetEntry : public QObject
    {
        Q_OBJECT
    public:
        RobotUnitLVL1ControllerClassesWidgetEntry(
            RobotUnitLVL1ControllerClassesWidget& parent,
            QTreeWidget& treeWidget,
            const  LVL1ControllerClassDescription& desc,
            RobotUnitInterfacePrx robotUnit
        );

        bool matchName(const QString& name);
        bool hasRemoteCreation();

    public slots:
        void setVisible(bool vis);

    private slots:
        void createCtrl();

    public:
        std::string className;
        QString classNameQStr;
        RobotUnitInterfacePrx robotUnit;
        QTreeWidgetItem* header {nullptr};
        QLineEdit* nameEdit {nullptr};
        WidgetDescription::DescribedWidgetBase* creator {nullptr};
    };
}
#endif
