/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * \package    RobotAPI::gui-plugins::RobotUnitPlugin
 * \author     Raphael ( ufdrv at student dot kit dot edu )
 * \date       2017
 * \copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#ifndef _ARMARX_RobotAPI_RobotUnitPlugin_RobotUnitLVL1ControllersWidget_H
#define _ARMARX_RobotAPI_RobotUnitPlugin_RobotUnitLVL1ControllersWidget_H

#include <mutex>

#include <QWidget>
#include <QLabel>
#include <QLineEdit>
#include <QHBoxLayout>
#include <QFormLayout>
#include <QPushButton>
#include <QTreeWidget>
#include <QTreeWidgetItem>
#include <QCheckBox>

#include <ArmarXCore/core/util/algorithm.h>
#include <ArmarXGui/libraries/WidgetDescription/WidgetDescription.h>
#include <ArmarXGui/libraries/VariantWidget/VariantWidget.h>
#include <RobotAPI/interface/units/RobotUnit/RobotUnitInterface.h>
#include <RobotAPI/interface/units/RobotUnit/LVL1Controller.h>

namespace Ui
{
    class RobotUnitLVL1ControllersWidget;
}

namespace armarx
{
    class RobotUnitLVL1ControllersWidget;
    class RobotUnitLVL1ControllersWidgetEntry;
    class RobotUnitLVL1ControllersWidgetRemoteFunction;

    class RobotUnitLVL1ControllersWidget : public QWidget
    {
        Q_OBJECT

    public:
        explicit RobotUnitLVL1ControllersWidget(QWidget* parent = 0);
        ~RobotUnitLVL1ControllersWidget();
        void lVl1ControllerStatusChanged(const LVL1ControllerStatus& status);
    public slots:
        virtual void lvl1ControllerCreated(std::string name);
        virtual void lvl1ControllerDeleted(std::string name);
        void reset(RobotUnitInterfacePrx ru);
    private slots:
        void lVl1ControllerStatusChanged();
    private:
        std::map<std::string, RobotUnitLVL1ControllersWidgetEntry*> entries;
        RobotUnitInterfacePrx robotUnit;
        mutable std::recursive_mutex mutex;
        Ui::RobotUnitLVL1ControllersWidget* ui;

        mutable std::recursive_mutex statusUpdatesMutex;
        std::map<std::string, LVL1ControllerStatus> statusUpdates;

    public:
        static constexpr int idxName       = 0;
        static constexpr int idxClass      = 1;
        static constexpr int idxActive     = 2;
        static constexpr int idxRequested  = 3;
        static constexpr int idxError      = 4;
        static constexpr int idxCtrlDev    = 5;
        static constexpr int idxCtrlMode   = 6;
        static constexpr int idxActivate   = 7;
        static constexpr int idxDeactivate = 8;
        static constexpr int idxDelete     = 9;
    };

    class RobotUnitLVL1ControllersWidgetEntry : public QObject
    {
        Q_OBJECT
    public:
        RobotUnitLVL1ControllersWidgetEntry(
            RobotUnitLVL1ControllersWidget& parent,
            QTreeWidget& treeWidget,
            const LVL1ControllerDescription& desc
        );

        void update(const LVL1ControllerStatus& status);

        bool matchName(const QString& name);
        bool matchClass(const QString& name);
        bool isActiveState(const QString& state);
        bool isRequestedState(const QString& state);
        bool isErrorState(const QString& state);
        bool matchDevice(const QString& dev);
        bool matchMode(const QString& mode);
        void deleteContent();

    public slots:
        void setVisible(bool vis);
    private slots:
        void activateController();
        void deactivateController();
        void deleteController();

        void hideDeviceList();

        void setDeviceListVisible(bool vis);
    private:
        QCheckBox* boxDev;
        QCheckBox* boxMod;
        QTreeWidgetItem* header;
        LVL1ControllerInterfacePrx controller;
        std::vector<QTreeWidgetItem*> devsToModes;

    };

    class RobotUnitLVL1ControllersWidgetRemoteFunction : public QWidget
    {
        Q_OBJECT
    public:
        RobotUnitLVL1ControllersWidgetRemoteFunction(const std::string& functionName, const LVL1ControllerInterfacePrx& ctrl, const WidgetDescription::WidgetPtr& w);
    private slots:
        void paramsChanged();
        void execFunction();
    private:
        std::string functionName;
        LVL1ControllerInterfacePrx ctrl;

        QCheckBox* execOnParamChange;
        WidgetDescription::DescribedWidgetBase* params;
    };
}
#endif
